function [i_trans, t_trans]=thickness_2D_hexagon_while(gamma, gammaI, kc, kt, Npr, Dpr, Jpr, I0,...
    lN, lD, lJ,nN, nD, nJ, Jpr_EC, A_Notch, A_Jagged, eps_ref,sig_ref, sFileName)

nf = 1.0;
lDf = 1.0;      % Lambda Delta of shifted Hill function [dimensionless]
lJf = 1.0;      % Lambda Jagged of shifted Hill function [dimensionless]

AsR = 1; % aspect ratio of cells computed as dy/dx

cell_size = 0.01; % mm
radius = 3.2; % mm

% -------------------------------------------------------------------------
% variables 
tmin = 0;  % start time
tmax = 2000; % end time    default 250
dt = 0.01; % time step (dt)
% ncells = 5; % number of cells 
ncombs = 100; % number of simulations - take average over 100 simulations

% -------------------------------------------------------------------------
% Dpr_range = 900:300:1800;
ncells_range_x = 1:100; % cells in the horizontal direction
ncells_y = 10;   % number of cells in the vertical direction
ncells_y_ini = ncells_y;

% N_total = zeros(length(ncells_range_x),ncells_range_x(end));
% D_total = zeros(length(ncells_range_x),ncells_range_x(end));
% J_total = zeros(length(ncells_range_x),ncells_range_x(end));
% I_total = zeros(length(ncells_range_x),ncells_range_x(end));

% Save number of states per length (multiple simulations)
nS = zeros(ncombs,1);
nSR = zeros(ncombs,1);
nR = zeros(ncombs,1);

% Save number of cell states and average protein levels for different lengths
ncellstates1 = zeros(length(ncells_range_x),4);
proteins_levels1 = zeros(length(ncells_range_x),8);

% Save average protein levels per cell for different lengths
N_mean = zeros(length(ncells_range_x),ncells_range_x(end));
D_mean = zeros(length(ncells_range_x),ncells_range_x(end));
J_mean = zeros(length(ncells_range_x),ncells_range_x(end));
I_mean = zeros(length(ncells_range_x),ncells_range_x(end));
I_end = zeros(length(ncells_range_x),ncells_range_x(end));
tic
for ppp = 1:length(ncells_range_x)
    ncells_x = ncells_range_x(ppp)
    
    thickness = ncells_x*cell_size;
    sigma = 16*radius/thickness;
    Nmf = exp(A_Notch*(eps_ref/sig_ref)*sigma);
    Jmf = exp(A_Jagged*(eps_ref/sig_ref)*sigma);
    Dmf = 1;
    
    t = zeros((tmax/dt),1);
    t(1) = tmin;
    
    % array with protein numbers in each cell at each time point 
    Notch = zeros((tmax/dt),ncells_x, ncells_y);
    Delta = zeros((tmax/dt),ncells_x, ncells_y);
    Jagged = zeros((tmax/dt),ncells_x, ncells_y);
    NICD = zeros((tmax/dt),ncells_x, ncells_y);
    
    % array with protein numbers in each cell at t = tmax for all
    N_combs = zeros(ncombs,ncells_x, ncells_y);
    D_combs = zeros(ncombs,ncells_x, ncells_y);
    J_combs = zeros(ncombs,ncells_x, ncells_y);
    I_combs = zeros(ncombs,ncells_x, ncells_y);
    
    for s = 1:ncombs
        % random initial condition for each cell, chosen between a certain
        % range (based on bifurgation plots Boareto)
        % same ranges as Boareto used were chosen
        rng('shuffle') % still random after restarting Matlab

        % value for N, D, J and I at tmin (initial value)
        Notch(1,:, :) = randi([0, 6000],ncells_x,ncells_y);
        Delta(1,:, :) = randi([0, 6000],ncells_x,ncells_y);
        Jagged(1, :, :) = randi([0, 6000],ncells_x,ncells_y);
        NICD(1, :, :) = randi([0, 600],ncells_x,ncells_y);
        
        i = 0;
        tol = 0.1;
        while (tol > 0.01 || dt*i < 250) && (dt*i < 3000)
            i = i + 1;
            % calculate Ne, De and Je for each cell
            % function Hillfunction = HS(I,I0,n,l)
            % function Neighbors = extp_dt(ncells,Notch,Delta,Jagged,NICD,I0,nf,lDf,lJf,i)
            Neighbors = extp_F_dt_oneway(ncells_x, ncells_y, Notch,Delta,Jagged,NICD,I0,nf,lDf,lJf,i,0,tmin,tmax,dt,Jpr_EC, AsR);
            NeD = Neighbors(1,:, :); % number of Notch molecules in neighboring cells
            NeJ = Neighbors(2,:, :); % number of Notch molecules in neighboring cells
            De = Neighbors(3,:, :); % number of Delta molecules in neighboring cells 
            Je = Neighbors(4,:, :); % number of Jagged molecules in neighboring cells
            
            t(i+1) = i*dt;
            
            % calculate number of proteins (N,D,J,I) at each time point
            % expicit solution t_new = t_old + differential equations
            % store number of proteins at the next time step in the next row
            Notch(i+1,:,:) = Notch(i,:,:) + dt*(Npr*Nmf*HS(NICD(i,: , : ),I0,nN,lN) - Notch(i,: , : ).*((kc*Delta(i,: , : ) + kt*De) ...
                .*HS(NICD(i,: , : ),I0,nf,lDf) + (kc*Jagged(i,: , : ) + kt*Je).*HS(NICD(i,: , : ),I0,nf,lJf)) - gamma*Notch(i,: , : ));
            Delta(i+1,: , : ) = Delta(i,: , : ) + dt*(Dpr*Dmf*HS(NICD(i,: , : ),I0,nD,lD) - Delta(i,: , : ).*(kc*Notch(i,: , : ) + kt*NeD) ...
                .*HS(NICD(i,: , : ),I0,nf,lDf) - gamma*Delta(i,: , : ));
            Jagged(i+1,: , : ) = Jagged(i,: , : ) + dt*(Jpr*Jmf*HS(NICD(i,: , : ),I0,nJ,lJ) - Jagged(i,: , : ).*(kc*Notch(i,: , : ) + kt*NeJ) ...
                .*HS(NICD(i,: , : ),I0,nf,lJf) - gamma*Jagged(i,: , : ));
            NICD(i+1,: , : ) = NICD(i,: , : ) + dt*(kt*Notch(i,: , : ).*(De.*HS(NICD(i,: , : ),I0,nf,lDf) + Je.*HS(NICD(i,: , : ),I0,nf,lJf)) ...
                - gammaI.*NICD(i,: , : ));
            tol = sum(sum(abs((kt*Notch(i,:,:).*(De.*HS(NICD(i,:,:),I0,nf,lDf) + Je.*HS(NICD(i,:,:),I0,nf,lJf)) ...
                 - gammaI.*NICD(i,:,:)))))/(ncells_x*ncells_y);
            
            for r = 1:ncells_x
                for kk = 1:ncells_y
                    if Notch(i+1,r, kk) < 0
                        Notch(i+1,r, kk) = 0;
                    end
                    if Delta(i+1,r, kk) < 0
                        Delta(i+1,r, kk) = 0;
                    end
                    if Jagged(i+1,r, kk) < 0
                        Jagged(i+1,r, kk) = 0;
                    end
                    if NICD(i+1,r, kk) < 0
                        NICD(i+1,r, kk) = 0;
                    end
                end
            end
        end
        
%         if ncells == ncells_range_x(end)
%             figure; hold on; plot(t,Notch(:,[1,10,50,ncells])); title('Notch');
%             figure; hold on; plot(t,Delta(:,[1,10,50,ncells])); title('Delta');
%             figure; hold on; plot(t,Jagged(:,[1,10,50,ncells])); title('Jagged');
%             figure; hold on; plot(t,NICD(:,[1,10,50,ncells])); title('NICD');
%         end 
        
        % save for each simulation the value of N, D, J and I at time = tmax
        % s is simulation number (s/ncombs)
        N_combs(s,:,:) = Notch(i+1,:,:);
        D_combs(s,:,:) = Delta(i+1,:,:);
        J_combs(s,:,:) = Jagged(i+1,:,:);
        I_combs(s,:,:) = NICD(i+1,:,:);
        
    end % end for s = 1:ncombs
    
    % store average protein levels per cell for each length
    N_mean(ppp,1:ncells_x, 1:ncells_y) = mean(N_combs,1);
    D_mean(ppp,1:ncells_x, 1:ncells_y) = mean(D_combs,1);
    J_mean(ppp,1:ncells_x, 1:ncells_y) = mean(J_combs,1);
    I_mean(ppp,1:ncells_x, 1:ncells_y) = mean(I_combs,1);
    I_end(ppp,1:ncells_x, 1:ncells_y) = I_combs(end,:,:);
    
    % STD?? N_std(ppp,1:cells) = std(N_combs,1)
%    N_std(ppp,1:ncells_x) = std(N_combs);
%    D_std(ppp,1:ncells_x) = std(D_combs);
%    J_std(ppp,1:ncells_x) = std(J_combs);
%    I_std(ppp,1:ncells_x) = std(I_combs);
    
    % store average + standard deviation protein level per length
    proteins_levels1(ppp,1) = ncells_x;
    proteins_levels1(ppp,2) = mean(mean(mean(N_combs)));
    proteins_levels1(ppp,3) = mean(mean(std(N_combs)));
    proteins_levels1(ppp,4) = mean(mean(mean(D_combs)));
    proteins_levels1(ppp,5) = mean(mean(std(D_combs)));
    proteins_levels1(ppp,6) = mean(mean(mean(J_combs)));
    proteins_levels1(ppp,7) = mean(mean(std(J_combs)));
    proteins_levels1(ppp,8) = mean(mean(mean(I_combs)));
    proteins_levels1(ppp,9) = mean(mean(std(I_combs)));
    
    % determine cell fate based on average NICD value
    cstate = 0.5*ones(ncombs,ncells_x, ncells_y);
    cstate(I_combs<100) = 0; % undifferentiated (Fleur: 145; Sandra: 100)
    cstate(I_combs>300) = 1; % default Fleur 300, Sandra 300
    
    % calculate number of differentiated and undifferentiated cells for each simulation
    for z = 1:ncombs 
        v = cstate(z,:, :);
        nS(z) = length(find(v == 0));
        nR(z) = length(find(v == 1));
        nSR(z) = length(find(v == 0.5));
    end
    
    % Calculate the average and std of the number of states for all simulations
    mean_nS = mean(nS);
    mean_nSR = mean(nSR);
    mean_nR = mean(nR);
    std_nS = std(nS);
    std_nSR = std(nSR);
    std_nR = std(nR);
    
    % mean values of ncombs
    ncellstates1(ppp,1) = ncells_x;
%     ncellstates1(ppp,2) = h;
%     ncellstates1(ppp,3) = T;
%     ncellstates1(ppp,4) = length(nS);
%     ncellstates1(ppp,5) = length(nSR);
%     ncellstates1(ppp,6) = length(nR);
%     ncellstates1(ppp,7) = length(nS)/ncells*100;
%     ncellstates1(ppp,8) = length(nSR)/ncells*100;
%     ncellstates1(ppp,9) = length(nR)/ncells*100;
    ncellstates1(ppp,4) = mean_nS;
    ncellstates1(ppp,5) = mean_nSR;
    ncellstates1(ppp,6) = mean_nR;
    ncellstates1(ppp,7) = std_nS;
    ncellstates1(ppp,8) = std_nSR;
    ncellstates1(ppp,9) = std_nR;    
    ncellstates1(ppp,10) = mean_nS/(ncells_x*ncells_y)*100;
    ncellstates1(ppp,11) = mean_nSR/(ncells_x*ncells_y)*100;
    ncellstates1(ppp,12) = mean_nR/(ncells_x*ncells_y)*100;
    ncellstates1(ppp,13) = std_nS/(ncells_x*ncells_y)*100;
    ncellstates1(ppp,14) = std_nSR/(ncells_x*ncells_y)*100;
    ncellstates1(ppp,15) = std_nR/(ncells_x*ncells_y)*100;
   
end % end for r = 1:length(Dpr_range)
toc

% Calculate transition thickness
nSR_tot = ncellstates1(:,11);
nSR_diff = zeros(size(nSR_tot,1),1);
for i = 2:length(nSR_diff)
    nSR_diff(i) = nSR_tot(i) - nSR_tot(i-1);
end
i_trans = find(nSR_diff == max(nSR_diff));
t_trans = (i_trans + 1)*cell_size;

save(sFileName, 'I_end', 'i_trans', 't_trans', 'ncells_y', 'ncells_y_ini',...
    'ncells_range_x', 'ncellstates1', 'proteins_levels1',...
    'N_mean', 'D_mean', 'J_mean', 'I_mean' );

end