%{
    % to produce the mex version: run the following commands at prompt:

    sArgs = {double(1), double(1), double(1),double(1), double(1), double(1), ...
        double(1), double(1), double(1), double(1),double(1), double(1), double(1), ...
        double(1), double(1),double(1), double(1), double(1),...
        double(1), double(1),double(1), double(1), double(1),...
        double(1), double(1), double(1),double(1), double(1)};
    codegen Notch_signaling_3Dartery -args sArgs

%}

function [N_combs, D_combs, J_combs, I_combs] = Notch_signaling_3Dartery(ncombs,...
    ncells_x, ncells_y, ncells_z, cell_size, radius, eps_ref, sig_ref, A_Notch, A_Jagged,...
    tmax, tmin, dt, Jpr_EC, Npr, Dpr, Jpr, lN, lD, lJ, nN, nD, nJ, gamma, gammaI,...
    kc, kt, I0)
   
    thickness = ncells_x*cell_size;
    sigma = 16*radius/thickness;
    Nmf = exp(A_Notch*(eps_ref/sig_ref)*sigma);
    Jmf = exp(A_Jagged*(eps_ref/sig_ref)*sigma);
    Dmf = 1;
    
    t = zeros((tmax/dt),1);
    t(1) = tmin;
    
    % array with protein numbers in each cell at each time point 
    Notch = zeros((tmax/dt),ncells_x, ncells_y, ncells_z);
    Delta = zeros((tmax/dt),ncells_x, ncells_y, ncells_z);
    Jagged = zeros((tmax/dt),ncells_x, ncells_y, ncells_z);
    NICD = zeros((tmax/dt),ncells_x, ncells_y, ncells_z);
    
    % array with protein numbers in each cell at t = tmax for all
    N_combs = zeros(ncombs,ncells_x, ncells_y, ncells_z);
    D_combs = zeros(ncombs,ncells_x, ncells_y, ncells_z);
    J_combs = zeros(ncombs,ncells_x, ncells_y, ncells_z);
    I_combs = zeros(ncombs,ncells_x, ncells_y, ncells_z);
    
    for s = 1:ncombs
        % random initial condition for each cell, chosen between a certain
        % range (based on bifurgation plots Boareto)
        % same ranges as Boareto used were chosen

        % value for N, D, J and I at tmin (initial value)
        Notch(1,:, :,:) = randi([0, 6000],ncells_x,ncells_y, ncells_z);
        Delta(1,:, :,:) = randi([0, 6000],ncells_x,ncells_y, ncells_z);
        Jagged(1, :, :,:) = randi([0, 6000],ncells_x,ncells_y, ncells_z);
        NICD(1, :, :,:) = randi([0, 600],ncells_x,ncells_y, ncells_z);
        
        i = 0;
        tol = 0.1;
        while (tol > 0.01 || dt*i < 250) && (dt*i < 3000)
            i = i + 1;
            % calculate Ne, De and Je for each cell
            % function Hillfunction = HS(I,I0,n,l)
            % function Neighbors = extp_dt(ncells,Notch,Delta,Jagged,NICD,I0,i)
            Neighbors = extp_F_dt_oneway(ncells_x, ncells_y, ncells_z, Notch,Delta,Jagged,i,Jpr_EC);
            Ne = Neighbors(1,:, :, :); % number of Notch molecules in neighboring cells
            De = Neighbors(2,:, :, :); % number of Delta molecules in neighboring cells 
            Je = Neighbors(3,:, :, :); % number of Jagged molecules in neighboring cells
            
            t(i+1) = i*dt;
            
            % calculate number of proteins (N,D,J,I) at each time point
            % expicit solution t_new = t_old + differential equations
            % store number of proteins at the next time step in the next row
            Notch(i+1,:,:, :) = Notch(i,:,:, :) + dt*(Npr*Nmf*HS(NICD(i,: , :, :),I0,nN,lN) - Notch(i,: , : , :).*((kc*Delta(i,: , : , :) + kt*De) ...
              + (kc*Jagged(i,: , : , :) + kt*Je)) - gamma*Notch(i,: , : , :));
            Delta(i+1,: , :, :) = Delta(i,: , :, :) + dt*(Dpr*Dmf*HS(NICD(i,: , :, :),I0,nD,lD) - Delta(i,: , :, :).*(kc*Notch(i,: , :, :) + kt*Ne) ...
                 - gamma*Delta(i,: , :, :));
            Jagged(i+1,: , :, :) = Jagged(i,: , :, :) + dt*(Jpr*Jmf*HS(NICD(i,: , :, :),I0,nJ,lJ) - Jagged(i,: , :, :).*(kc*Notch(i,: , :, :) + kt*Ne) ...
                 - gamma*Jagged(i,: , :, :));
            NICD(i+1,: , :, :) = NICD(i,: , :, :) + dt*(kt*Notch(i,: , :, :).*(De + Je) ...
                - gammaI.*NICD(i,: , :, :));
            tol = sum(sum(sum(abs((kt*Notch(i,:,:,:).*(De + Je) ...
                 - gammaI.*NICD(i,:,:,:))),2),3),4)/(ncells_x*ncells_y*ncells_z);
             
            for r = 1:ncells_x
                for kk = 1:ncells_y
                    for abc=1:ncells_z
                        if Notch(i+1,r, kk,abc) < 0
                            Notch(i+1,r, kk,abc) = 0;
                        end
                        if Delta(i+1,r, kk,abc) < 0
                            Delta(i+1,r, kk,abc) = 0;
                        end
                        if Jagged(i+1,r, kk,abc) < 0
                            Jagged(i+1,r, kk,abc) = 0;
                        end
                        if NICD(i+1,r, kk,abc) < 0
                            NICD(i+1,r, kk,abc) = 0;
                        end
                    end
                end
            end
        end
        
        % save for each simulation the value of N, D, J and I at time = tmax
        % s is simulation number (s/ncombs)
        N_combs(s,:,:,:) = Notch(i+1,:,:,:);
        D_combs(s,:,:,:) = Delta(i+1,:,:,:);
        J_combs(s,:,:,:) = Jagged(i+1,:,:,:);
        I_combs(s,:,:,:) = NICD(i+1,:,:,:);
        
    end % end for s = 1:ncombs

end

%%

function HillFunction = HS(I,I0,n,l)
% HillFunction = l + (1-l)*(1/(1+(I/I0)^n));
HillFunction = l + (1-l).*(1./(1+(I./I0).^n));
end

%%

function Neighbors = extp_F_dt_oneway(ncells_x,ncells_y,ncells_z,Notch,Delta,Jagged,i,Jpr_EC)

%handles.value meegeven, if h == 1 initial, h == 2 several, etc

% Edited by Sandra Loerakker: polarization of Jagged

% Edited by Tommaso Ristori: 10 November 2010
% - Bidirectional signal not clustered, with vertical symmetry

% endothelial cells are to be identified with the first row

% sum external proteins of Notch (Nt)        
% ------------------------------------------------------------------------------------------------- 
% Notch
N = zeros(ncells_x+2, ncells_y+2, ncells_z+2);
N(2:(ncells_x+1), 2:(ncells_y+1), 2:(ncells_z+1)) = Notch(i,:, :, :);
% simmetry in the direction y
N(2:(ncells_x+1), 1, 2:(ncells_z+1)) = N(2:(ncells_x+1), ncells_y+1, 2:(ncells_z+1));
N(2:(ncells_x+1), ncells_y+2, 2:(ncells_z+1)) = N(2:(ncells_x+1), 2, 2:(ncells_z+1));
% simmetry in the direction z
N(2:(ncells_x+1), 2:(ncells_y+1), 1) = N(2:(ncells_x+1), 2:(ncells_y+1), ncells_z+1);
N(2:(ncells_x+1), 2:(ncells_y+1), ncells_z+2) = N(2:(ncells_x+1), 2:(ncells_y+1), 2);

N_l = N(1:ncells_x, 2:(ncells_y+1),  2:(ncells_z+1)); % N[i-1, j, k] 
N_r = N(3:ncells_x+2, 2:(ncells_y+1),  2:(ncells_z+1)); % N[i+1, j, k]
N_down = N(2:ncells_x+1,1:ncells_y,  2:(ncells_z+1)); % N[i, j-1, k]
N_up = N(2:ncells_x+1,3:ncells_y+2,  2:(ncells_z+1)); % N[i, j+1, k]
N_forward = N(2:ncells_x+1,2:(ncells_y+1),  1:ncells_z); % N[i, j, k-1]
N_backwards = N(2:ncells_x+1,2:(ncells_y+1),  3:ncells_z+2); % N[i, j, k+1]

% Delta
D = zeros(ncells_x+2, ncells_y+2, ncells_z+2);
D(2:(ncells_x+1), 2:(ncells_y+1), 2:(ncells_z+1)) = Delta(i,:, :, :);
% simmetry in the direction y
D(2:(ncells_x+1), 1, 2:(ncells_z+1)) = D(2:(ncells_x+1), ncells_y+1, 2:(ncells_z+1));
D(2:(ncells_x+1), ncells_y+2, 2:(ncells_z+1)) = D(2:(ncells_x+1), 2, 2:(ncells_z+1));
% simmetry in the direction z
D(2:(ncells_x+1), 2:(ncells_y+1), 1) = D(2:(ncells_x+1), 2:(ncells_y+1), ncells_z+1);
D(2:(ncells_x+1), 2:(ncells_y+1), ncells_z+2) = D(2:(ncells_x+1), 2:(ncells_y+1), 2);

D_l = D(1:ncells_x, 2:(ncells_y+1),  2:(ncells_z+1)); % N[i-1, j, k] 
D_r = D(3:ncells_x+2, 2:(ncells_y+1),  2:(ncells_z+1)); % N[i+1, j, k]
D_down = D(2:ncells_x+1,1:ncells_y,  2:(ncells_z+1)); % N[i, j-1, k]
D_up = D(2:ncells_x+1,3:ncells_y+2,  2:(ncells_z+1)); % N[i, j+1, k]
D_forward = D(2:ncells_x+1,2:(ncells_y+1),  1:ncells_z); % N[i, j, k-1]
D_backwards = D(2:ncells_x+1,2:(ncells_y+1),  3:ncells_z+2); % N[i, j, k+1]

% Jagged

% continuous pulse EC
J = zeros(ncells_x+2, ncells_y+2, ncells_z+2);
%J(1,ncells_x+2, :) = 0;
J(1, :, :) = Jpr_EC; 
J(2:(ncells_x+1), 2:(ncells_y+1), 2:(ncells_z+1)) = Jagged(i,:, :, :);
% simmetry in the direction y
J(2:(ncells_x+1), 1, 2:(ncells_z+1)) = J(2:(ncells_x+1), ncells_y+1, 2:(ncells_z+1));
J(2:(ncells_x+1), ncells_y+2, 2:(ncells_z+1)) = J(2:(ncells_x+1), 2, 2:(ncells_z+1));
% simmetry in the direction z
J(2:(ncells_x+1), 2:(ncells_y+1), 1) = J(2:(ncells_x+1), 2:(ncells_y+1), ncells_z+1);
J(2:(ncells_x+1), 2:(ncells_y+1), ncells_z+2) = J(2:(ncells_x+1), 2:(ncells_y+1), 2);

J_l = J(1:ncells_x, 2:(ncells_y+1),  2:(ncells_z+1)); % N[i-1, j, k] 
J_r = J(3:ncells_x+2, 2:(ncells_y+1),  2:(ncells_z+1)); % N[i+1, j, k]
J_down = J(2:ncells_x+1,1:ncells_y,  2:(ncells_z+1)); % N[i, j-1, k]
J_up = J(2:ncells_x+1,3:ncells_y+2,  2:(ncells_z+1)); % N[i, j+1, k]
J_forward = J(2:ncells_x+1,2:(ncells_y+1),  1:ncells_z); % N[i, j, k-1]
J_backwards = J(2:ncells_x+1,2:(ncells_y+1),  3:ncells_z+2); % N[i, j, k+1]

% calculate Ne, De and Je for each cell
Neighbors = zeros(3,ncells_x, ncells_y, ncells_z);
Neighbors(1,:, :,:) = (N_l + N_r + N_up + N_down+ N_forward + N_backwards)/6; % Ne     
Neighbors(2,:, :,:) = (D_l + D_r + D_up + D_down+ D_forward + D_backwards)/6; % De
Neighbors(3,:, :,:) = (J_l + J_r + J_up + J_down + J_forward + J_backwards)/6; % Je
end