
rng('shuffle') % still random after restarting Matlab

gamma = 0.1;    % [1/h] degradation of Notch (N), Delta (D), Jagged (J)
gammaI = 0.5;   % [1/h] degradation of NICD (I)
kc = 5e-04;     % [1/h] strength cis-inhibition
kt = 2.5e-05;   % [1/h] strengtth transactivation
Npr = 1400;     % [number of proteins] production rate Notch in molec
Dpr = 100;      % [number of proteins] production rate Delta in molec
Jpr = 1600;     % [number of proteins] production rate Delta in molec
I0 = 200;       % [number of proteins] threshold of the Hill function in molec
lN = 2.0;       % Lambda Notch of shifted Hill function [dimensionless]
lD = 0.0;       % Lambda Delta of shifted Hill function [dimensionless]
lJ = 2.0;       % Lambda Jagged of shifted Hill function [dimensionless]
nN = 2.0;       % Hill Notch coefficients [dimensionless]
nD = 2.0;       % Hill Delta coefficients [dimensionless]
nJ = 5.0;       % Hill Jagged coefficients [dimensionless]
Jpr_EC = 4000;
A_Notch = -5.79;
A_Jagged = -4.17;
eps_ref = 0.075;
sig_ref = 107;

cell_size = 0.01; % mm
radius = 3.2; % mm

% -------------------------------------------------------------------------
% variables 
tmin = 0;  % start time
tmax = 2000; % end time
dt = 0.01; % time step (dt)
% ncells = 5; % number of cells 
ncombs = 100; % number of simulations - take average over 100 simulations

% -------------------------------------------------------------------------
% Dpr_range = 900:300:1800;
ncells_range_x = 1:100; % cells in the horizontal direction
ncells_y = 10;   % number of cells in the vertical direction
ncells_z = 4;

% N_total = zeros(length(ncells_range_x),ncells_range_x(end));
% D_total = zeros(length(ncells_range_x),ncells_range_x(end));
% J_total = zeros(length(ncells_range_x),ncells_range_x(end));
% I_total = zeros(length(ncells_range_x),ncells_range_x(end));

% Save number of states per length (multiple simulations)
nS = zeros(ncombs,1);
nSR = zeros(ncombs,1);
nR = zeros(ncombs,1);

% Save number of cell states and average protein levels for different lengths
ncellstates1 = zeros(length(ncells_range_x),4);
proteins_levels1 = zeros(length(ncells_range_x),8);

% Save average protein levels per cell for different lengths
N_mean = zeros(length(ncells_range_x),ncells_range_x(end), ncells_y, ncells_z);
D_mean = zeros(length(ncells_range_x),ncells_range_x(end), ncells_y, ncells_z);
J_mean = zeros(length(ncells_range_x),ncells_range_x(end), ncells_y, ncells_z);
I_mean = zeros(length(ncells_range_x),ncells_range_x(end), ncells_y, ncells_z);
I_end = zeros(length(ncells_range_x),ncells_range_x(end), ncells_y, ncells_z);

for ppp = 1:length(ncells_range_x)
    tic
    ncells_x = ncells_range_x(ppp)
    
    [N_combs, D_combs, J_combs, I_combs] = Notch_signaling_3Dartery_mex( ncombs,...
    ncells_x, ncells_y, ncells_z, cell_size, radius, eps_ref, sig_ref, A_Notch, A_Jagged,...
    tmax, tmin, dt, Jpr_EC, Npr, Dpr, Jpr, lN, lD, lJ, nN, nD, nJ, gamma, gammaI,...
    kc, kt, I0);
    
    % store average protein levels per cell for each length
    N_mean(ppp,1:ncells_x, 1:ncells_y, 1:ncells_z) = mean(N_combs,1);
    D_mean(ppp,1:ncells_x, 1:ncells_y,1:ncells_z) = mean(D_combs,1);
    J_mean(ppp,1:ncells_x, 1:ncells_y,1:ncells_z) = mean(J_combs,1);
    I_mean(ppp,1:ncells_x, 1:ncells_y,1:ncells_z) = mean(I_combs,1);
    I_end(ppp,1:ncells_x, 1:ncells_y,1:ncells_z) = I_combs(end,:,:,:);
 
  
    % store average + standard deviation protein level per length
    proteins_levels1(ppp,1) = ncells_x;
    proteins_levels1(ppp,2) = mean(mean(mean(mean(N_combs))));
    proteins_levels1(ppp,3) = mean(mean(mean(std(N_combs))));
    proteins_levels1(ppp,4) = mean(mean(mean(mean(D_combs))));
    proteins_levels1(ppp,5) = mean(mean(mean(std(D_combs))));
    proteins_levels1(ppp,6) = mean(mean(mean(mean(J_combs))));
    proteins_levels1(ppp,7) = mean(mean(mean(std(J_combs))));
    proteins_levels1(ppp,8) = mean(mean(mean(mean(I_combs))));
    proteins_levels1(ppp,9) = mean(mean(mean(std(I_combs))));
    
    % determine cell fate based on average NICD value
    cstate = 0.5*ones(ncombs,ncells_x, ncells_y, ncells_z);
    cstate(I_combs<100) = 0; % undifferentiated (Fleur: 145; Sandra: 100)
    cstate(I_combs>300) = 1; % default Fleur 300, Sandra 300
    
    % calculate number of differentiated and undifferentiated cells for each simulation
    for z = 1:ncombs 
        v = cstate(z,:, :, :);
        nS(z) = length(find(v == 0));
        nR(z) = length(find(v == 1));
        nSR(z) = length(find(v == 0.5));
    end
    
    % Calculate the average and std of the number of states for all simulations
    mean_nS = mean(nS);
    mean_nSR = mean(nSR);
    mean_nR = mean(nR);
    std_nS = std(nS);
    std_nSR = std(nSR);
    std_nR = std(nR);
    
    % mean values of ncombs
    ncellstates1(ppp,1) = ncells_x;

    ncellstates1(ppp,4) = mean_nS;
    ncellstates1(ppp,5) = mean_nSR;
    ncellstates1(ppp,6) = mean_nR;
    ncellstates1(ppp,7) = std_nS;
    ncellstates1(ppp,8) = std_nSR;
    ncellstates1(ppp,9) = std_nR;    
    ncellstates1(ppp,10) = mean_nS/(ncells_x*ncells_y*ncells_z)*100;
    ncellstates1(ppp,11) = mean_nSR/(ncells_x*ncells_y*ncells_z)*100;
    ncellstates1(ppp,12) = mean_nR/(ncells_x*ncells_y*ncells_z)*100;
    ncellstates1(ppp,13) = std_nS/(ncells_x*ncells_y*ncells_z)*100;
    ncellstates1(ppp,14) = std_nSR/(ncells_x*ncells_y*ncells_z)*100;
    ncellstates1(ppp,15) = std_nR/(ncells_x*ncells_y*ncells_z)*100;
   toc
end % end for r = 1:length(ncells_range_x)


% Calculate transition thickness
nSR_tot = ncellstates1(:,11);
nSR_diff = zeros(size(nSR_tot,1),1);
for i = 2:length(nSR_diff)
    nSR_diff(i) = nSR_tot(i) - nSR_tot(i-1);
end
i_trans = find(nSR_diff == max(nSR_diff));
t_trans = (i_trans + 1)*cell_size;

save('artery_3D', 'I_end', 'i_trans', 't_trans', 'ncells_y',...
    'ncells_range_x', 'ncellstates1', 'proteins_levels1',...
    'N_mean', 'D_mean', 'J_mean', 'I_mean' );